# Generated by Django 4.2.2 on 2023-06-08 12:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cms_qe_table", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="tablepluginmodel",
            name="columns",
            field=models.JSONField(default=dict, verbose_name="Columns to show"),
        ),
        migrations.AlterField(
            model_name="tablepluginmodel",
            name="filter",
            field=models.JSONField(default=dict, verbose_name="Filter"),
        ),
    ]



from django import forms
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from .models import User
from .utils import smtp_server_accepts_email_address


class CmsQeAuthUserForm(forms.ModelForm):

    def clean(self):
        field_name = "email"
        if self.cleaned_data.get(field_name) and self.cleaned_data.get("is_active") and \
                self.cleaned_data.get("is_staff"):
            try:
                smtp_server_accepts_email_address(self.cleaned_data[field_name])
            except ValidationError as err:
                msg = _("Enter a valid email or don't enter any.")
                if "email" in self._errors:
                    self._errors[field_name].append(err)
                    self._errors[field_name].append(msg)
                else:
                    self._errors[field_name] = self.error_class([err, msg])
        return self.cleaned_data


class CmsQeAuthUserAdmin(UserAdmin):
    form = CmsQeAuthUserForm


admin.register(User)(CmsQeAuthUserAdmin)



from haystack.forms import ModelSearchForm

from .query import HaystackSearchQuerySet


# https://github.com/django-haystack/django-haystack/blob/v3.2.1/haystack/forms.py#L96
class HaystackSearchForm(ModelSearchForm):
    """Haystack ModelSearchForm."""

    def __init__(self, *args, **kwargs):
        self.searchqueryset = kwargs.pop("searchqueryset", None)
        self.load_all = kwargs.pop("load_all", False)
        super().__init__(*args, **kwargs)
        self.searchqueryset = HaystackSearchQuerySet()

    def search(self):
        if not self.is_valid():
            return self.no_query_found()

        if not self.cleaned_data.get("q"):
            return self.no_query_found()

        sqs = self.searchqueryset.post_process_query(self.cleaned_data["q"])

        if self.load_all:
            sqs = sqs.load_all()

        return sqs



from django import forms

from cms_qe_newsletter.models import Subscriber


class SubscriberForm(forms.ModelForm):
    """
    Subscribers form validates subscriber data.
    """

    class Meta:
        model = Subscriber
        fields = ('email', 'first_name', 'last_name')

    def __init__(self, fullname_require, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fullname_require = fullname_require
        if fullname_require:
            self.fields['first_name'].required = True
            self.fields['last_name'].required = True

    # pylint: disable=arguments-differ
    def save(self, commit=True, mailing_lists=None):
        """
        Add a new subscriber to all mail lists which was selected in the plugin settings.
        """
        data = self.cleaned_data
        email = data['email']
        first_name = data['first_name']
        last_name = data['last_name']
        if mailing_lists is not None:
            for mailing_list in mailing_lists:
                if not Subscriber.objects.filter(email=email, mailing_list=mailing_list).count():
                    Subscriber.objects.create(
                        mailing_list=mailing_list,
                        email=email,
                        first_name=first_name,
                        last_name=last_name,
                    )



# Generated by Django 4.2.13 on 2024-06-04 08:08

import cms_qe_plugins.validators
import django.core.serializers.json
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Iframe",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="%(app_label)s_%(class)s",
                        serialize=False,
                        to="cms.cmsplugin",
                    ),
                ),
                (
                    "src",
                    models.CharField(
                        help_text="Specifies the address of the document to embed in the &lt;iframe&gt;.",
                        max_length=255,
                        validators=[cms_qe_plugins.validators.validate_url_or_path],
                    ),
                ),
                (
                    "allow",
                    models.CharField(
                        blank=True,
                        help_text="Specifies a feature policy for the &lt;iframe&gt;. Form more see <a href='https://developer.mozilla.org/en-US/docs/Web/HTML/Element/iframe#allow' target='_blank'>IFrame attributes.</a>.",
                        max_length=255,
                        null=True,
                    ),
                ),
                (
                    "allow_full_screen",
                    models.BooleanField(
                        blank=True,
                        help_text="Set to true if the &lt;iframe&gt; can activate fullscreen mode by calling the requestFullscreen() method.",
                        null=True,
                    ),
                ),
                (
                    "allow_payment_request",
                    models.BooleanField(
                        blank=True,
                        help_text="Set to true if a cross-origin &lt;iframe&gt; should be allowed to invoke the Payment Request API.",
                        null=True,
                    ),
                ),
                (
                    "height",
                    models.IntegerField(
                        blank=True,
                        help_text="Specifies the height of an &lt;iframe&gt;. Default height is 150 pixels.",
                        null=True,
                    ),
                ),
                (
                    "width",
                    models.IntegerField(
                        blank=True,
                        help_text="Specifies the width of an &lt;iframe&gt;. Default width is 300 pixels.",
                        null=True,
                    ),
                ),
                (
                    "loading",
                    models.CharField(
                        blank=True,
                        choices=[("", ""), ("eager", "eager"), ("lazy", "lazy")],
                        default=None,
                        help_text="Specifies whether a browser should load an iframe immediately or to defer loading of iframes until some conditions are met.",
                        max_length=255,
                        null=True,
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        blank=True,
                        help_text="Specifies the name of an &lt;iframe&gt;.",
                        max_length=255,
                        null=True,
                    ),
                ),
                (
                    "referrer_policy",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("", ""),
                            ("no-referrer", "no-referrer"),
                            (
                                "no-referrer-when-downgrade",
                                "no-referrer-when-downgrade",
                            ),
                            ("origin", "origin"),
                            ("origin-when-cross-origin", "origin-when-cross-origin"),
                            ("unsafe-url", "unsafe-url"),
                            ("same-origin", "same-origin"),
                            ("strict-origin", "strict-origin"),
                            (
                                "strict-origin-when-cross-origin",
                                "strict-origin-when-cross-origin",
                            ),
                        ],
                        default=None,
                        help_text="Specifies which referrer information to send when fetching the iframe.",
                        max_length=50,
                        null=True,
                    ),
                ),
                (
                    "sandbox",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("", ""),
                            ("allow-forms", "allow-forms"),
                            ("allow-pointer-lock", "allow-pointer-lock"),
                            ("allow-popups", "allow-popups"),
                            ("allow-same-origin", "allow-same-origin"),
                            ("allow-scripts", "allow-scripts"),
                            ("allow-top-navigation", "allow-top-navigation"),
                        ],
                        default=None,
                        help_text="Enables an extra set of restrictions for the content in an &lt;iframe&gt;.",
                        max_length=50,
                        null=True,
                    ),
                ),
                (
                    "src_doc",
                    models.TextField(
                        blank=True,
                        help_text="Specifies the HTML content of the page to show in the &lt;iframe&gt;.",
                        null=True,
                    ),
                ),
                (
                    "attributes",
                    models.JSONField(
                        blank=True,
                        encoder=django.core.serializers.json.DjangoJSONEncoder,
                        help_text='More attributes as JSON data. E.g. {"data-name": "value", "id": 42}',
                        null=True,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Link",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="%(app_label)s_%(class)s",
                        serialize=False,
                        to="cms.cmsplugin",
                    ),
                ),
                (
                    "href",
                    models.CharField(
                        help_text="Specifies the location of the linked document.",
                        max_length=255,
                        validators=[cms_qe_plugins.validators.validate_url_or_path],
                    ),
                ),
                (
                    "rel",
                    models.CharField(
                        choices=[
                            ("alternate", "alternate"),
                            ("author", "author"),
                            ("dns-prefetch", "dns-prefetch"),
                            ("help", "help"),
                            ("icon", "icon"),
                            ("license", "license"),
                            ("next", "next"),
                            ("pingback", "pingback"),
                            ("preconnect", "preconnect"),
                            ("prefetch", "prefetch"),
                            ("preload", "preload"),
                            ("prerender", "prerender"),
                            ("prev", "prev"),
                            ("search", "search"),
                            ("stylesheet", "stylesheet"),
                        ],
                        default="stylesheet",
                        help_text="Required. Specifies the relationship between the current document and the linked document.",
                        max_length=30,
                    ),
                ),
                (
                    "cross_origin",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("", ""),
                            ("anonymous", "anonymous"),
                            ("use-credentials", "use-credentials"),
                        ],
                        default=None,
                        help_text="Specifies how the element handles cross-origin requests.",
                        max_length=15,
                        null=True,
                    ),
                ),
                (
                    "hreflang",
                    models.CharField(
                        blank=True,
                        help_text="Specifies the language of the text in the linked document.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "media",
                    models.CharField(
                        blank=True,
                        help_text="Specifies on what device the linked document will be displayed.",
                        max_length=255,
                        null=True,
                    ),
                ),
                (
                    "referrer_policy",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("", ""),
                            ("no-referrer", "no-referrer"),
                            (
                                "no-referrer-when-downgrade",
                                "no-referrer-when-downgrade",
                            ),
                            ("origin", "origin"),
                            ("origin-when-cross-origin", "origin-when-cross-origin"),
                            ("unsafe-url", "unsafe-url"),
                        ],
                        default=None,
                        help_text="Specifies which referrer to use when fetching the resource.",
                        max_length=50,
                        null=True,
                    ),
                ),
                (
                    "sizes",
                    models.CharField(
                        blank=True,
                        help_text='Specifies the size of the linked resource. Only for rel="icon".',
                        max_length=255,
                        null=True,
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        blank=True,
                        help_text="Defines a preferred or an alternate stylesheet.",
                        max_length=255,
                        null=True,
                    ),
                ),
                (
                    "type",
                    models.CharField(
                        blank=True,
                        help_text="Specifies the media type of the linked document. E.g. 'text/css'.",
                        max_length=255,
                        null=True,
                    ),
                ),
                (
                    "attributes",
                    models.JSONField(
                        blank=True,
                        encoder=django.core.serializers.json.DjangoJSONEncoder,
                        help_text='More attributes as JSON data. E.g. {"data-name": "value", "id": 42}',
                        null=True,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("cms.cmsplugin",),
        ),
        migrations.CreateModel(
            name="Script",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="%(app_label)s_%(class)s",
                        serialize=False,
                        to="cms.cmsplugin",
                    ),
                ),
                (
                    "src",
                    models.CharField(
                        help_text="Specifies the location of the linked script.",
                        max_length=255,
                        validators=[cms_qe_plugins.validators.validate_url_or_path],
                    ),
                ),
                (
                    "asyncf",
                    models.BooleanField(
                        default=False,
                        help_text="Specifies that the script is downloaded in parallel to parsing the page, and executed as soon as it is available (before parsing completes) (only for external scripts)",
                        verbose_name="async",
                    ),
                ),
                (
                    "cross_origin",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("", ""),
                            ("anonymous", "anonymous"),
                            ("use-credentials", "use-credentials"),
                        ],
                        default=None,
                        help_text="Sets the mode of the request to an HTTP CORS Request.",
                        max_length=15,
                        null=True,
                    ),
                ),
                (
                    "defer",
                    models.BooleanField(
                        default=False,
                        help_text="Specifies that the script is downloaded in parallel to parsing the page, and executed after the page has finished parsing (only for external scripts)",
                    ),
                ),
                (
                    "integrity",
                    models.CharField(
                        blank=True,
                        help_text="Allows a browser to check the fetched script to ensure that the code is never loaded if the source has been manipulated",
                        max_length=255,
                        null=True,
                    ),
                ),
                (
                    "nomodule",
                    models.BooleanField(
                        default=False,
                        help_text="Specifies that the script should not be executed in browsers supporting ES2015 modules.",
                    ),
                ),
                (
                    "referrer_policy",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("", ""),
                            ("no-referrer", "no-referrer"),
                            (
                                "no-referrer-when-downgrade",
                                "no-referrer-when-downgrade",
                            ),
                            ("origin", "origin"),
                            ("origin-when-cross-origin", "origin-when-cross-origin"),
                            ("unsafe-url", "unsafe-url"),
                            ("same-origin", "same-origin"),
                            ("strict-origin", "strict-origin"),
                            (
                                "strict-origin-when-cross-origin",
                                "strict-origin-when-cross-origin",
                            ),
                        ],
                        default=None,
                        help_text="Specifies which referrer to use when fetching the resource.",
                        max_length=50,
                        null=True,
                    ),
                ),
                (
                    "type",
                    models.CharField(
                        blank=True,
                        help_text="Specifies the media type of the script.",
                        max_length=255,
                        null=True,
                    ),
                ),
                (
                    "attributes",
                    models.JSONField(
                        blank=True,
                        encoder=django.core.serializers.json.DjangoJSONEncoder,
                        help_text='More attributes as JSON data. E.g. {"data-name": "value", "id": 42}',
                        null=True,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("cms.cmsplugin",),
        ),
    ]



from django.conf import settings
from django.contrib.sites.models import Site
from django.core.cache import cache
from django.http import HttpRequest, HttpResponse
from django.views import View

from cms_qe.signals import run_subprocess


class HealthCheckView(View):
    """Health check View."""

    def get(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        """Apply HTTP GET. Used by uwsgi in docker."""
        # Check database connection.
        site = Site.objects.first()
        if not site.domain:
            raise RuntimeError('db')
        # Check cache.
        key = 'health-check-test'
        cache.set(key, 'OK', 2)
        if cache.get(key) != 'OK':
            raise RuntimeError('cache')
        return HttpResponse("OK", content_type="text/plain")


class ReloadSiteView(View):
    """Reload site View."""

    def get(self, request: HttpRequest, *args, **kwargs) -> HttpResponse:
        """Run subprocess defined in settings."""
        if request.user.is_superuser:
            if settings.RELOAD_SITE:
                msg = str(run_subprocess(settings.RELOAD_SITE))
            else:
                msg = "SKIP: No settings.RELOAD_SITE."
        else:
            msg = "ERROR: User is not is_superuser."
        return HttpResponse(msg, content_type="text/plain")



import json
import logging
from typing import Any

from constance import config
from django.http import Http404, HttpRequest, HttpResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.views.generic import View

__all__ = ('csp_report',)


@csrf_exempt
@require_POST
def csp_report(request: HttpRequest) -> HttpResponse:
    """
    View handling reports by CSP headers. When there is problem by CSP,
    then browser fire request to this view with JSON data describing
    problem. It's simply just logged as warning for later analyzing.
    """
    data = request.read()
    data = json.loads(str(data, 'utf8', 'replace'))
    logging.warning(data)
    return HttpResponse('OK')


class SecurityTxtView(View):
    """Provide file security.txt."""

    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        """Serve content of security.txt."""
        if not config.SECURITY_TXT_CONTENT:
            raise Http404()
        return HttpResponse(config.SECURITY_TXT_CONTENT, content_type="text/plain")



from aldryn_search.views import AldrynSearchView

from cms_qe.haystack.forms import HaystackSearchForm


class SiteSearchView(AldrynSearchView):  # pylint: disable=too-many-ancestors
    """Site Search View."""

    form_class = HaystackSearchForm
    template_name = "cms_qe/search_result.html"



from cms.models.pluginmodel import CMSPlugin
from django.core.serializers.json import DjangoJSONEncoder
from django.db import models

from .attributes import CROSS_ORIGINS, LOADING, REFERRER_POLICY, REL, SANDBOX, SCRIPT_REFERRER_POLICY
from .utils import make_choices, make_opt_choices
from .validators import validate_url_or_path


class Script(CMSPlugin):
    """HTML Script tag."""

    src = models.CharField(
        max_length=255, validators=[validate_url_or_path],
        help_text="Specifies the location of the linked script.")
    asyncf = models.BooleanField(
        default=False,
        verbose_name='async',
        help_text='Specifies that the script is downloaded in parallel to parsing the page, and executed as soon as it '
                  'is available (before parsing completes) (only for external scripts)'
        )
    cross_origin = models.CharField(
        max_length=15, null=True, blank=True, choices=make_opt_choices(CROSS_ORIGINS), default=None,
        help_text="Sets the mode of the request to an HTTP CORS Request.")
    defer = models.BooleanField(
        default=False,
        help_text='Specifies that the script is downloaded in parallel to parsing the page, and executed after '
                  'the page has finished parsing (only for external scripts)'
    )
    integrity = models.CharField(
        max_length=255, null=True, blank=True,
        help_text='Allows a browser to check the fetched script to ensure that the code is never loaded '
                  'if the source has been manipulated'
    )
    nomodule = models.BooleanField(
        default=False,
        help_text='Specifies that the script should not be executed in browsers supporting ES2015 modules.'
    )
    referrer_policy = models.CharField(
        max_length=50, null=True, blank=True, choices=make_opt_choices(SCRIPT_REFERRER_POLICY), default=None,
        help_text="Specifies which referrer to use when fetching the resource.")
    type = models.CharField(
        max_length=255, null=True, blank=True,
        help_text="Specifies the media type of the script.")
    attributes = models.JSONField(
        null=True, blank=True, encoder=DjangoJSONEncoder,
        help_text='More attributes as JSON data. E.g. {"data-name": "value", "id": 42}')

    def __str__(self):
        return self.src


class Link(CMSPlugin):
    """HTML Link tag."""

    href = models.CharField(
        max_length=255, validators=[validate_url_or_path],
        help_text="Specifies the location of the linked document.")
    rel = models.CharField(
        max_length=30, choices=make_choices(REL), default='stylesheet',
        help_text="Required. Specifies the relationship between the current document and the linked document.")
    cross_origin = models.CharField(
        max_length=15, null=True, blank=True, choices=make_opt_choices(CROSS_ORIGINS), default=None,
        help_text="Specifies how the element handles cross-origin requests.")
    hreflang = models.CharField(
        max_length=30, null=True, blank=True, help_text="Specifies the language of the text in the linked document.")
    media = models.CharField(
        max_length=255, null=True, blank=True,
        help_text="Specifies on what device the linked document will be displayed.")
    referrer_policy = models.CharField(
        max_length=50, null=True, blank=True, choices=make_opt_choices(REFERRER_POLICY), default=None,
        help_text="Specifies which referrer to use when fetching the resource.")
    sizes = models.CharField(
        max_length=255, null=True, blank=True,
        help_text='Specifies the size of the linked resource. Only for rel="icon".')
    title = models.CharField(
        max_length=255, null=True, blank=True,
        help_text="Defines a preferred or an alternate stylesheet.")
    type = models.CharField(
        max_length=255, null=True, blank=True,
        help_text="Specifies the media type of the linked document. E.g. 'text/css'.")
    attributes = models.JSONField(
        null=True, blank=True, encoder=DjangoJSONEncoder,
        help_text='More attributes as JSON data. E.g. {"data-name": "value", "id": 42}')

    def __str__(self):
        return self.href


class Iframe(CMSPlugin):
    """HTML Iframe tag."""

    src = models.CharField(
        max_length=255, validators=[validate_url_or_path],
        help_text="Specifies the address of the document to embed in the &lt;iframe&gt;.")
    allow = models.CharField(
        max_length=255, null=True, blank=True,
        help_text="Specifies a feature policy for the &lt;iframe&gt;. Form more see "
                  "<a href='https://developer.mozilla.org/en-US/docs/Web/HTML/Element/iframe#allow' target='_blank'>"
                  "IFrame attributes.</a>.")
    allow_full_screen = models.BooleanField(
        null=True, blank=True,
        help_text='Set to true if the &lt;iframe&gt; can activate fullscreen mode by calling the requestFullscreen() '
                  'method.'
    )
    allow_payment_request = models.BooleanField(
        null=True, blank=True,
        help_text='Set to true if a cross-origin &lt;iframe&gt; should be allowed to invoke the Payment Request API.'
    )
    height = models.IntegerField(
        null=True, blank=True,
        help_text='Specifies the height of an &lt;iframe&gt;. Default height is 150 pixels.')
    width = models.IntegerField(
        null=True, blank=True,
        help_text='Specifies the width of an &lt;iframe&gt;. Default width is 300 pixels.')
    loading = models.CharField(
        max_length=255, null=True, blank=True, choices=make_opt_choices(LOADING), default=None,
        help_text="Specifies whether a browser should load an iframe immediately or to defer loading of iframes "
                  "until some conditions are met.")
    name = models.CharField(
        max_length=255, null=True, blank=True,
        help_text="Specifies the name of an &lt;iframe&gt;.")
    referrer_policy = models.CharField(
        max_length=50, null=True, blank=True, choices=make_opt_choices(SCRIPT_REFERRER_POLICY), default=None,
        help_text="Specifies which referrer information to send when fetching the iframe.")
    sandbox = models.CharField(
        max_length=50, null=True, blank=True, choices=make_opt_choices(SANDBOX), default=None,
        help_text="Enables an extra set of restrictions for the content in an &lt;iframe&gt;.")
    src_doc = models.TextField(
        null=True, blank=True, help_text="Specifies the HTML content of the page to show in the &lt;iframe&gt;."
    )
    attributes = models.JSONField(
        null=True, blank=True, encoder=DjangoJSONEncoder,
        help_text='More attributes as JSON data. E.g. {"data-name": "value", "id": 42}')

    def __str__(self):
        return self.src



# Generated by Django 4.2.2 on 2023-06-09 08:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cms_qe_auth", "0003_auto_20210420_1050"),
    ]

    operations = [
        migrations.AlterField(
            model_name="user",
            name="first_name",
            field=models.CharField(
                blank=True, max_length=150, verbose_name="first name"
            ),
        ),
        migrations.AlterField(
            model_name="user",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
    ]



from django.core.validators import RegexValidator, URLValidator, _lazy_re_compile
from django.utils.translation import gettext_lazy as _

path_validator = RegexValidator(
    _lazy_re_compile(r"^/\w+"),
    message=_("Enter a valid URL or path."),
    code="invalid",
)


def validate_url_or_path(value: str) -> None:
    """Validate URL or path."""
    if value[:1] == '/':
        return path_validator(value)
    return URLValidator()(value)



import os

from .dev import *  # noqa: F403

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, '..', 'db_selenium.sqlite3'),  # noqa: F405
    }
}



from django.apps import AppConfig


class CmsQEConfig(AppConfig):
    name = 'cms_qe'

    def ready(self):
        from cms.signals import urls_need_reloading  # pylint: disable=import-outside-toplevel

        from .signals import reload_site  # pylint: disable=import-outside-toplevel

        urls_need_reloading.connect(reload_site)



# Generated by Django 4.2.2 on 2023-06-09 09:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("cms_qe_auth", "0004_alter_user_first_name_alter_user_id"),
    ]

    operations = [
        migrations.DeleteModel(
            name="Group",
        ),
    ]



CROSS_ORIGINS = (
    'anonymous',
    'use-credentials',
)

LOADING = (
    'eager',
    'lazy',
)

SANDBOX = (
    'allow-forms',
    'allow-pointer-lock',
    'allow-popups',
    'allow-same-origin',
    'allow-scripts',
    'allow-top-navigation',
)

REFERRER_POLICY = (
    'no-referrer',
    'no-referrer-when-downgrade',
    'origin',
    'origin-when-cross-origin',
    'unsafe-url',
)

SCRIPT_REFERRER_POLICY = REFERRER_POLICY + (
    'same-origin',
    'strict-origin',
    'strict-origin-when-cross-origin',
)

REL = (
    'alternate',
    'author',
    'dns-prefetch',
    'help',
    'icon',
    'license',
    'next',
    'pingback',
    'preconnect',
    'prefetch',
    'preload',
    'prerender',
    'prev',
    'search',
    'stylesheet',
)



def make_choices(items: tuple[str, ...]) -> list[tuple[str, str]]:
    """Make field required choices."""
    return [(item, item) for item in items]


def make_opt_choices(items: tuple[str, ...]) -> list[tuple[str, str]]:
    """Make field optional choices."""
    return [('', '')] + make_choices(items)



from .dev import *

INSTALLED_APPS += [  # noqa: F405
    # Aldryn News&Blog
    'aldryn_apphooks_config',
    'aldryn_common',
    'aldryn_categories',
    'aldryn_newsblog',
    'aldryn_people',
    'aldryn_translation_tools',
    'parler',
    'sortedm2m',
    'taggit',
]



import re

from django import template

register = template.Library()


@register.filter
def add_str(value, arg):
    """
    Same as :py:func:`django.template.defaultfilters.add` but always convert to ``str``.
    """
    return str(value) + str(arg)


@register.filter
def split_by_delimiter(value, delimiter):
    """
    Split the value by the delimiter.
    """
    return value.split(delimiter)


@register.filter
def get_sequence_item(value, position):
    """
    Get sequence item at the position.
    """
    try:
        return value[position]
    except IndexError:
        return ""


@register.filter
def matches_pattern(value, pattern):
    """
    The value matches pattern.
    """
    if value is None:
        return False
    return bool(re.match(pattern, value))



"""
Base settings for Django app.
"""

# Default primary key field type
# https://docs.djangoproject.com/en/4.2/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

SITE_ID = 1

INTERNAL_IPS = ['127.0.0.1']

META_USE_SITES = True
META_SITE_PROTOCOL = 'https'

INSTALLED_APPS = [
    # This app. :-)
    # It is holding at the top of the list, so that allow rewrite the templates in third side applications.
    'cms_qe',
    'cms_qe_auth',
    'cms_qe_breadcrumb',
    'cms_qe_i18n',
    'cms_qe_menu',
    'cms_qe_newsletter',
    'cms_qe_table',
    'cms_qe_video',
    'cms_qe_analytical',
    'cms_qe_plugins',

    # Must be before django.contrib.admin.
    'djangocms_admin_style',

    # Django's defaults.
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',  # Needed by Django CMS.
    'django.contrib.sitemaps',

    # Django CMS's core modules.
    'cms',
    'menus',
    'treebeard',  # Tree structure of pages and plugins.
    'sekizai',  # Static file management.

    # Other Django CMS's useful modules.
    'djangocms_text_ckeditor',
    'djangocms_googlemap',

    # Django Filer's modules.
    'filer',
    'easy_thumbnails',

    # Other Django Files's useful modules.
    'djangocms_file',
    'djangocms_icon',
    'djangocms_picture',

    # Optional django CMS Frontend modules
    'djangocms_frontend',
    'djangocms_frontend.contrib.accordion',
    'djangocms_frontend.contrib.alert',
    'djangocms_frontend.contrib.badge',
    'djangocms_frontend.contrib.card',
    'djangocms_frontend.contrib.carousel',
    'djangocms_frontend.contrib.collapse',
    'djangocms_frontend.contrib.content',
    'djangocms_frontend.contrib.grid',
    'djangocms_frontend.contrib.jumbotron',
    'djangocms_frontend.contrib.link',
    'djangocms_frontend.contrib.listgroup',
    'djangocms_frontend.contrib.media',
    'djangocms_frontend.contrib.image',
    'djangocms_frontend.contrib.tabs',
    'djangocms_frontend.contrib.utilities',

    # Other Django's modules.
    'axes',
    'constance',
    'constance.backends.database',
    'import_export',
    'mailqueue',

    # Aldryn forms
    'aldryn_forms',
    'aldryn_forms.contrib.email_notifications',
    'captcha',

    # Serach engine
    'haystack',
    'standard_form',
    'spurl',
    'aldryn_search',
]

MIDDLEWARE = [
    # Must be the first. Cache is more important, second one is only
    # for development auto-reload also after apphook changes.
    'django.middleware.cache.UpdateCacheMiddleware',
    'cms.middleware.utils.ApphookReloadMiddleware',

    # Django's defaults.
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',  # Locale is mandatory by Django CMS.
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',

    # Extra Django's middlewares.
    'django.middleware.common.BrokenLinkEmailsMiddleware',

    # Django CMS's core middlewares.
    'cms.middleware.user.CurrentUserMiddleware',
    'cms.middleware.page.CurrentPageMiddleware',
    'cms.middleware.toolbar.ToolbarMiddleware',
    'cms.middleware.language.LanguageCookieMiddleware',

    # CMS QE middleware.
    'cms_qe.middleware.page_status_code.restore_status_for_cached_error_page',

    # Security middleware.
    'axes.middleware.AxesMiddleware',
    'csp.middleware.CSPMiddleware',

    # Must be the last.
    'django.middleware.cache.FetchFromCacheMiddleware',
]

# Reload site. For example ['uwsgi', '--reload', '/var/run/uwsgi.pid'] or ['touch', 'manage.py'].
RELOAD_SITE: list[str] = []



import logging
from collections import namedtuple

from django.contrib import admin
from django.core.exceptions import FieldDoesNotExist
from django.http import Http404, HttpResponse
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _
from import_export import fields, resources, widgets
from tablib import Dataset

ExportType = namedtuple("ExportType", ("mimetype", "label"))

# Map from supported export types to their content type used for file to download.
# {extension: (mimetype, label), ...}
EXPORT_TYPES_TO_CONTENT_TYPES = {
    'csv': ExportType('text/csv', _('Export selected as CSV')),
    'tsv': ExportType('text/tab-separated-values', _('Export selected as TSV (Tab-Separated Values)')),
    'xls': ExportType('application/vnd.ms-excel', _('Export selected as XLS')),
    'xlsx': ExportType('application/vnd.ms-excel', _('Export selected as XLSX')),
    'ods': ExportType('application/vnd.oasis.opendocument.spreadsheet', _('Export selected as ODS')),
    'json': ExportType('application/json', _('Export selected as JSON')),
    'yaml': ExportType('text/plain', _('Export selected as YAML')),
    'html': ExportType('text/html', _('Export selected as HTML')),
    'latex': ExportType('application/x-latex', _('Export selected as LATEX (LaTeX Source Document)')),
    'rst': ExportType('text/x-rst', _('Export selected as RST (reStructuredText)')),
}

logger = logging.getLogger(__name__)


def get_supported_export_types():
    """
    Get only the export types supported by this operating system.
    """
    dataset = Dataset()
    available_types = {}
    for ext, item in EXPORT_TYPES_TO_CONTENT_TYPES.items():
        try:
            getattr(dataset, ext)
            available_types[ext] = item
        except (ImportError, AttributeError) as error:
            logger.info(error)
    return available_types


def register_export_action(export_type, label):
    """
    Helper to register export action to specific type. It dynamically
    creates action function and register it as ``admin.site.add_action``
    with passed label.
    """

    # pylint: disable=unused-argument
    # Warning: Some admin pages can add more arguments to actions.
    def wrapper(modeladmin, request, queryset, *args, **kwargs):
        return export_file(export_type, modeladmin, queryset)

    wrapper.__name__ = f'export_selected_objects_as_{export_type}'
    wrapper.short_description = label

    admin.site.add_action(wrapper, wrapper.__name__)

    return wrapper


def export_file(export_type, modeladmin, queryset):
    """
    Same as :any:`cms_qe.export.export_data` but wraps it into
    :any:`django.http.HttpResponse` as file to download.
    """
    export_types = get_supported_export_types()
    if export_type not in export_types:
        raise Http404(f'Type {export_type} is not supported')

    content_type = export_types[export_type].mimetype
    data = export_data(export_type, modeladmin, queryset)
    response = HttpResponse(data, content_type=content_type)
    response['Content-Disposition'] = f'attachment; filename="export.{export_type}"'
    return response


def export_data(export_type, modeladmin, queryset):
    """
    Export data as ``export_type``. Supported are only those from
    function ``get_supported_export_types``.
    """
    if hasattr(modeladmin, "export_data"):
        return modeladmin.export_data(export_type, queryset)

    model_fields = queryset.model._meta.get_fields()

    class ObjectResourceMeta(resources.ModelDeclarativeMetaclass):
        """
        When model has field with choices, we want to export verbose names
        instead of keys in database. For that on resource object has to be
        field with it's list of choices. ``ModelResource`` uses metaclass
        to collect those data so we need to override that metaclass and
        pass it manually. Assign it to class after creation is late.
        """
        def __new__(cls, name, bases, attrs):  # pylint: disable=bad-mcs-classmethod-argument
            for field in model_fields:
                choices = getattr(field, 'choices', None)
                if not choices:
                    continue
                attrs[field.name] = fields.Field(
                    attribute=field.name,  # Warning: attribute has to be here.
                    widget=ChoicesWidget(choices),
                )

            # ModelAdmin can specify extra fields which could be property on
            # the model or method on the ModelAdmin. Because those fields
            # are visible on the web to the user, it's good to include them
            # in the export as well.
            for field_name in modeladmin.list_display:
                if field_name in attrs or field_name[:2] == '__':
                    continue
                attrs[field_name] = AdminField(
                    attribute=field_name,
                    modeladmin=modeladmin,
                )

            return super().__new__(cls, name, bases, attrs)

    # pylint: disable=too-few-public-methods
    class ObjectResource(resources.ModelResource, metaclass=ObjectResourceMeta):
        class Meta:
            model = queryset.model

            # After adding some extra fields, it will break order.
            # Django's ``get_fields`` keeps order but we need to exclude
            # all auto created back references.
            # Warning: ID is also auto created.
            export_order = [
                field.name
                for field in model_fields
                if not field.auto_created or field.name == 'id'
            ]

        def get_export_headers(self) -> list:
            """
            As header use verbose name which is better than database name.
            """
            return [self.get_model_field_verbose_name(field) for field in self.get_export_fields()]

        def get_model_field_verbose_name(self, field) -> str:
            name = self.get_field_name(field)
            try:
                field = queryset.model._meta.get_field(name)
            except FieldDoesNotExist:
                pass
            else:
                name = force_str(field.verbose_name)
            return name

        def export_field(self, field, obj):
            value = super().export_field(field, obj)
            if '__proxy__' in value.__class__.__name__:
                value = force_str(value)
            return value

    dataset = ObjectResource().export(queryset)
    return getattr(dataset, export_type)


class AdminField(fields.Field):
    """
    Field to support including extra fields defined in the variable
    `ModelAdmin.list_display`. Original `fields.Field` is capable
    to get property so we need to only add support of extra fields
    defined on the ModelAdmin itself.
    """

    def __init__(self, *args, **kwds):
        modeladmin = kwds.pop('modeladmin')
        self.get_modeladmin = lambda: modeladmin
        super().__init__(*args, **kwds)

    def get_value(self, obj):
        admin_property = getattr(self.get_modeladmin(), self.attribute, None)
        if admin_property:
            return admin_property(obj)
        return super().get_value(obj)


# Taken from https://github.com/django-import-export/django-import-export/issues/525#issuecomment-303046691
class ChoicesWidget(widgets.Widget):
    """
    Widget that uses choice display values in place of database values
    """

    # pylint:disable=unused-argument
    def __init__(self, choices, *args, **kwargs):
        """
        Creates a self.choices dict with a key, display value, and value,
        db value, e.g. {'Chocolate': 'CHOC'}
        """
        self.choices = dict(choices)
        self.revert_choices = {v: k for k, v in self.choices.items()}

    # pylint: disable=keyword-arg-before-vararg
    def clean(self, value, row=None, *args, **kwargs):
        """Returns the db value given the display value"""
        return self.revert_choices.get(value, value) if value else None

    def render(self, value, obj=None):
        """Returns the display value given the db value"""
        return self.choices.get(value, '')



"""
URL Configuration
https://docs.djangoproject.com/en/1.11/topics/http/urls/
"""

from django.urls import path

from .views import update_lists

urlpatterns = [
    path('cms-qe/newsletter/sync-lists', update_lists, name='sync_mailing_lists'),
]



import logging
import subprocess
import time
from multiprocessing import Lock, Process
from typing import Any, Optional, Union

from django.conf import settings
from django.db.models import Model

DelayType = Union[int, float]
ArgsType = Union[tuple[str, ...], list[str]]

logger = logging.getLogger(__name__)
lock = Lock()


def run_subprocess(args: ArgsType, delay: Optional[DelayType] = None) -> Optional[subprocess.CompletedProcess]:
    """Run the system command in subprocess."""
    if delay is not None:
        time.sleep(delay)
    logger.info('subprocess.run(%s)', args)
    lock.acquire()
    try:
        return subprocess.run(args, check=False)
    except Exception as err:  # pylint: disable=broad-exception-caught
        logger.error(str(err))
    finally:
        lock.release()
    return None


def run_process(args: ArgsType, delay: Optional[DelayType] = None) -> Process:
    """Run the process and no wait for the result."""
    proc = Process(target=run_subprocess, args=(args, delay))
    proc.start()
    return proc


def reload_site(sender: Optional[Model], **kwargs: dict[str, Any]) -> None:  # pylint: disable=unused-argument
    "Reload the site."
    if settings.RELOAD_SITE:
        run_process(settings.RELOAD_SITE, 3)



"""
Use in MIDDLEWARE:
    cms_qe.middleware.page_status_code.restore_status_for_cached_error_page
"""
from typing import Callable

from django.http import HttpRequest, HttpResponse

from ..constants import QE_STATUS_CODE


def restore_status_for_cached_error_page(get_response: Callable) -> Callable:
    """Set middleware for restoring status code of error page."""

    def middleware(request: HttpRequest) -> HttpResponse:
        """Restore page status code saved into the page headers."""
        response = get_response(request)
        status_code = response.get(QE_STATUS_CODE) if hasattr(response, 'headers') else None
        if status_code is not None:
            response.status_code = int(status_code)
            del response[QE_STATUS_CODE]
        return response

    return middleware



from haystack.backends.whoosh_backend import WhooshEngine, WhooshSearchBackend, WhooshSearchQuery
from haystack.constants import DJANGO_CT, DJANGO_ID, ID
from haystack.exceptions import SearchBackendError
from haystack.fields import CharField
from whoosh.analysis import CharsetFilter, StemmingAnalyzer
from whoosh.analysis.analyzers import CompositeAnalyzer
from whoosh.fields import BOOLEAN, DATETIME, ID as WHOOSH_ID, IDLIST, KEYWORD, NGRAM, NGRAMWORDS, NUMERIC, TEXT, Schema
from whoosh.support.charset import accent_map


# https://github.com/django-haystack/django-haystack/blob/v3.2.1/haystack/backends/whoosh_backend.py#L77
class AnalyzerWhooshSearchBackend(WhooshSearchBackend):
    """WhooshSearchBackend width analyzer."""

    # This is a copy of parent class function modified by _get_analyzer.
    # https://github.com/django-haystack/django-haystack/blob/v3.2.1/haystack/backends/whoosh_backend.py#L168
    def build_schema(self, fields):  # pylint: disable=too-many-branches
        schema_fields = {
            ID: WHOOSH_ID(stored=True, unique=True),
            DJANGO_CT: WHOOSH_ID(stored=True),
            DJANGO_ID: WHOOSH_ID(stored=True),
        }
        # Grab the number of keys that are hard-coded into Haystack.
        # We'll use this to (possibly) fail slightly more gracefully later.
        initial_key_count = len(schema_fields)
        content_field_name = ""

        for _, field_class in fields.items():
            if field_class.is_multivalued:
                if field_class.indexed is False:
                    schema_fields[field_class.index_fieldname] = IDLIST(
                        stored=True, field_boost=field_class.boost
                    )
                else:
                    schema_fields[field_class.index_fieldname] = KEYWORD(
                        stored=True,
                        commas=True,
                        scorable=True,
                        field_boost=field_class.boost,
                    )
            elif field_class.field_type in ["date", "datetime"]:
                schema_fields[field_class.index_fieldname] = DATETIME(
                    stored=field_class.stored, sortable=True
                )
            elif field_class.field_type == "integer":
                schema_fields[field_class.index_fieldname] = NUMERIC(
                    stored=field_class.stored,
                    numtype=int,
                    field_boost=field_class.boost,
                )
            elif field_class.field_type == "float":
                schema_fields[field_class.index_fieldname] = NUMERIC(
                    stored=field_class.stored,
                    numtype=float,
                    field_boost=field_class.boost,
                )
            elif field_class.field_type == "boolean":
                # Field boost isn't supported on BOOLEAN as of 1.8.2.
                schema_fields[field_class.index_fieldname] = BOOLEAN(
                    stored=field_class.stored
                )
            elif field_class.field_type == "ngram":
                schema_fields[field_class.index_fieldname] = NGRAM(
                    minsize=3,
                    maxsize=15,
                    stored=field_class.stored,
                    field_boost=field_class.boost,
                )
            elif field_class.field_type == "edge_ngram":
                schema_fields[field_class.index_fieldname] = NGRAMWORDS(
                    minsize=2,
                    maxsize=15,
                    at="start",
                    stored=field_class.stored,
                    field_boost=field_class.boost,
                )
            else:
                schema_fields[field_class.index_fieldname] = TEXT(
                    stored=True,
                    # analyzer=field_class.analyzer or StemmingAnalyzer(),  Original.
                    analyzer=_get_analyzer(field_class),  # Modification.
                    field_boost=field_class.boost,
                    sortable=True,
                )

            if field_class.document is True:
                content_field_name = field_class.index_fieldname
                schema_fields[field_class.index_fieldname].spelling = True

        # Fail more gracefully than relying on the backend to die if no fields
        # are found.
        if len(schema_fields) <= initial_key_count:
            raise SearchBackendError(
                "No fields were found in any search_indexes. Please correct this before attempting to search."
            )

        return (content_field_name, Schema(**schema_fields))

    def build_search_kwargs(self, **kwargs):  # pylint: disable=W0221
        """Build search kwargs."""
        # A convenience method most backends should include in order to make extension easier.
        return {}

    def extract_file_contents(self, file_obj):
        """Extract file contents."""


def _get_analyzer(field_class: CharField) -> CompositeAnalyzer:
    """Get analyzer."""
    analyzer = StemmingAnalyzer()
    if field_class is not None and hasattr(field_class, 'analyzer') and field_class.analyzer is not None:
        analyzer = field_class.analyzer
    if field_class.index_fieldname in ['text', 'description', 'title']:
        analyzer |= CharsetFilter(accent_map)
    return analyzer


class AnalyzerWhooshEngine(WhooshEngine):
    backend = AnalyzerWhooshSearchBackend
    query = WhooshSearchQuery



# Generated by Django 2.2.13 on 2021-04-20 10:50

import django.contrib.auth.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cms_qe_auth', '0002_auto_20171208_0915'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='group',
            managers=[
                ('objects', django.contrib.auth.models.GroupManager()),
            ],
        ),
        migrations.AlterField(
            model_name='user',
            name='last_name',
            field=models.CharField(blank=True, max_length=150, verbose_name='last name'),
        ),
    ]



"""
Configuration for development.

Disables all security options.
"""

import os
from pathlib import Path

from .base import *  # noqa: F401,F403 pylint: disable=wildcard-import,unused-wildcard-import

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.11/howto/deployment/checklist/

DEBUG = True

META_SITE_PROTOCOL = 'http'

SECRET_KEY = '^xzhq0*q1+t0*ihq^^1wuyj3i%y#(38b7d-vlpkm-d(=!^uk6x'

SESSION_COOKIE_SECURE = False

SECURE_HSTS_SECONDS = 0

ALLOWED_HOSTS = ['localhost', '127.0.0.1']

INSTALLED_APPS += [  # noqa: F405
    'debug_toolbar',
    'django_extensions',
]

ROOT_URLCONF = "cms_qe.urls"

MIDDLEWARE += [  # noqa: F405
    'debug_toolbar.middleware.DebugToolbarMiddleware',
]

site_resolver = Path(__file__).resolve()

PROJECT_DIR = site_resolver.parent.parent.parent

STATIC_ROOT = os.path.join(PROJECT_DIR, 'staticfiles')

# Caching
# https://docs.djangoproject.com/en/1.11/topics/cache/

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.filebased.FileBasedCache',
        'LOCATION': os.path.join(PROJECT_DIR, 'django_cache'),
    }
}

# Database
# https://docs.djangoproject.com/en/1.11/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(PROJECT_DIR, 'db.sqlite3'),
        'TEST': {
            'NAME': ':memory:',
        },
    }
}

EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'



from haystack.constants import FILTER_SEPARATOR
from haystack.query import SearchQuerySet

from .inputs import PostProcessQuery


class HaystackSearchQuerySet(SearchQuerySet):
    """Haystack SearchQuerySet."""

    def post_process_query(self, query_string, fieldname=f"content{FILTER_SEPARATOR}contains"):
        """Query contains value."""
        kwargs = {fieldname: PostProcessQuery(query_string)}
        return self.filter(**kwargs)



import unicodedata

from haystack.utils.highlighting import Highlighter


def strip_accents(text: str) -> str:
    """Strip accents."""
    return "".join([unicodedata.normalize("NFKD", char)[0] for char in text])


class HaystackHighlighter(Highlighter):
    """Haystack Highlighter."""

    # https://github.com/django-haystack/django-haystack/blob/v3.2.1/haystack/utils/highlighting.py#L32
    def find_highlightable_words(self):
        # Use a set so we only do this once per unique word.
        word_positions = {}

        # Pre-compute the length.
        end_offset = len(self.text_block)
        lower_text_block = self.text_block.lower()
        ascii_text_block = strip_accents(lower_text_block)

        for word in self.query_words:
            if word not in word_positions:
                word_positions[word] = []

            start_offset = 0

            while start_offset < end_offset:
                next_offset = lower_text_block.find(word, start_offset, end_offset)
                if next_offset == -1:
                    next_offset = ascii_text_block.find(word, start_offset, end_offset)

                # If we get a -1 out of find, it wasn't found. Bomb out and
                # start the next word.
                if next_offset == -1:
                    break

                word_positions[word].append(next_offset)
                start_offset = next_offset + len(word)

        return word_positions

    # pylint: disable=too-many-locals
    def render_html(self, highlight_locations=None, start_offset=None, end_offset=None):
        # Start by chopping the block down to the proper window.
        text = self.text_block[start_offset:end_offset]

        # Invert highlight_locations to a location -> term list
        term_list = []

        for term, locations in highlight_locations.items():
            term_list += [(loc - start_offset, term) for loc in locations]

        loc_to_term = sorted(term_list)

        # Prepare the highlight template
        if self.css_class:
            hl_start = f'<{self.html_tag} class="{self.css_class}">'
        else:
            hl_start = f"<{self.html_tag}>"

        hl_end = f"</{self.html_tag}>"

        # Copy the part from the start of the string to the first match,
        # and there replace the match with a highlighted version.
        highlighted_chunk = ""
        matched_so_far = 0
        prev = 0
        prev_str = ""

        for cur, cur_str in loc_to_term:
            # This can be in a different case than cur_str
            actual_term = text[cur:cur + len(cur_str)]

            # Handle incorrect highlight_locations by first checking for the term
            actual_term_lower = actual_term.lower()
            if actual_term_lower == cur_str or strip_accents(actual_term_lower) == cur_str:
                if cur < prev + len(prev_str):
                    continue

                highlighted_chunk += (
                    text[prev + len(prev_str):cur] + hl_start + actual_term + hl_end
                )
                prev = cur
                prev_str = cur_str

                # Keep track of how far we've copied so far, for the last step
                matched_so_far = cur + len(actual_term)

        # Don't forget the chunk after the last term
        highlighted_chunk += text[matched_so_far:]

        if start_offset > 0:
            highlighted_chunk = f"...{highlighted_chunk}"

        if end_offset < len(self.text_block):
            highlighted_chunk = f"{highlighted_chunk}..."

        return highlighted_chunk



"""
Settings providing authentication options.
"""

AUTH_USER_MODEL = 'cms_qe_auth.User'

LOGIN_REDIRECT_URL = '/'

LOGIN_URL = '/auth/login'

CMS_QE_AUTH_ENABLED = False  # Enable cms_qe_auth registration.

ALDRYN_FORMS_EMAIL_AVAILABILITY_CHECKER_FNC = 'cms_qe_auth.utils.smtp_server_accepts_email_address'
ALDRYN_FORMS_EMAIL_AVAILABILITY_CHECKER_CLASS = 'cms_qe_auth.utils.SMTPCheckRecipient'



# Generated by Django 4.2.2 on 2023-06-08 11:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="TablePluginModel",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="%(app_label)s_%(class)s",
                        serialize=False,
                        to="cms.cmsplugin",
                    ),
                ),
                (
                    "table",
                    models.CharField(max_length=100, verbose_name="Table to show"),
                ),
                (
                    "columns",
                    models.JSONField(default="", verbose_name="Columns to show"),
                ),
                ("filter", models.JSONField(default="", verbose_name="Filter")),
                (
                    "paging_show",
                    models.BooleanField(default=True, verbose_name="Show paging"),
                ),
                (
                    "paging_per_page",
                    models.IntegerField(
                        default=20, verbose_name="How many items per page when paging"
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("cms.cmsplugin",),
        ),
    ]



from django.conf import settings
from django.urls import path
from django.views.static import serve

from cms_qe.urls import handler403, handler404, handler500, handler503, urlpatterns


def serve_favicon(request):
    """Serve favicon.ico."""
    return serve(request, 'favicon.ico', settings.STATIC_ROOT)


if settings.DEBUG:
    urlpatterns += [
        path('favicon.ico', serve_favicon),
    ]



"""
URL Configuration
https://docs.djangoproject.com/en/1.11/topics/http/urls/
"""

from cms.sitemaps import CMSSitemap
from django.apps import apps
from django.conf import settings
from django.conf.urls.i18n import i18n_patterns
from django.conf.urls.static import static
from django.contrib import admin
from django.contrib.sitemaps.views import sitemap
from django.urls import include, path
from django.views.i18n import JavaScriptCatalog

from cms_qe.views.maintenance import HealthCheckView, ReloadSiteView
from cms_qe.views.search_result import SiteSearchView
from cms_qe.views.security import SecurityTxtView

from . import views

__all__ = (
    'handler403',
    'handler404',
    'handler500',
    'handler503',
    'urlpatterns',
)

# pylint: disable=invalid-name
handler403 = 'cms_qe.views.handler403'
handler404 = 'cms_qe.views.handler404'
handler500 = 'cms_qe.views.handler500'
handler503 = 'cms_qe.views.handler503'

urlpatterns = [
    path('', include('filer.server.urls')),
    path('csp-report', views.csp_report),
    path('', include('cms_qe_table.urls')),
    path('', include('cms_qe_newsletter.urls')),
    path('sitemap.xml', sitemap, {'sitemaps': {'cmspages': CMSSitemap}}),
    path('api/monitoring', views.get_monitoring),
    path('.well-known/security.txt', SecurityTxtView.as_view(), name='security-txt'),
    path('site-search-result/', SiteSearchView.as_view(), name='site-search-result'),
    path("healthcheck/", HealthCheckView.as_view(), name='healthcheck'),  # Used by uwsgi in docker.
    path("superuser/reload-site/", ReloadSiteView.as_view(), name='reload-site'),
]

# django-simple-captcha
if apps.is_installed("captcha"):
    urlpatterns += [
        path('captcha/', include('captcha.urls')),  # Aldryn forms
    ]

# During development is error page replaced by Django error page with debug info.
# This is registration special URLs for testing error pages in dev mode.
if settings.DEBUG:
    import debug_toolbar

    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
    urlpatterns += [
        path('__debug__/', include(debug_toolbar.urls)),
        path('403/', views.handler403),
        path('404/', views.handler404),
        path('503/', views.handler503),
    ]

# Django CMS has to be the last one because it will consume all URLs.
if settings.CMS_QE_AUTH_ENABLED:
    urlpatterns += i18n_patterns(
        path('jsi18n/', JavaScriptCatalog.as_view(), name='javascript-catalog'),
        path('jsi18n-aldryn-forms/', JavaScriptCatalog.as_view(packages=['aldryn_forms']), name='js-aldryn-forms'),
        path('admin/', admin.site.urls),
        path('auth/', include('cms_qe_auth.urls')),
        path('', include('cms.urls')),
    )
else:
    urlpatterns += i18n_patterns(
        path('jsi18n/', JavaScriptCatalog.as_view(), name='javascript-catalog'),
        path('jsi18n-aldryn-forms/', JavaScriptCatalog.as_view(packages=['aldryn_forms']), name='js-aldryn-forms'),
        path('admin/', admin.site.urls),
        path('', include('cms.urls')),
    )



import setuptools


setuptools.setup(
    name='django-cms-qe',
    version='3.4.1',
    packages=setuptools.find_packages(exclude=[
        '*.tests',
        '*.tests.*',
        'tests.*',
        'tests',
        'test_utils.*',
        'test_utils',
        '*.migrations',
        '*.migrations.*',
    ]),
    include_package_data=True,
    description=(
        'Django CMS Quick & Easy provides all important modules to run new page without'
        'a lot of coding. Aims to do it very easily and securely.'
    ),
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    url='https://websites.pages.nic.cz/django-cms-qe',
    author='CZ.NIC, z.s.p.o.',
    author_email='kontakt@nic.cz',
    license='BSD License',

    python_requires='>=3.9',  # https://docs.python.org/3.9/library/typing.html#module-contents
    install_requires=[
        'easy-thumbnails[svg]',
        'djangocms-frontend~=1.1',  # https://github.com/django-cms/djangocms-frontend/blob/1.1.4/setup.py#L6-L15
        'django-csp~=3.7',
        'djangocms-picture~=4.0',
        'django-axes~=6.0',
        'django-constance[database]~=2.9',
        'djangocms-file~=3.0',
        'django-import-export~=3.2',
        'django-mail-queue~=3.2',
        'djangocms-icon~=2.0',
        'djangocms-googlemap~=2.0',
        'django-tablib~=3.2',  # Used by cms_qe/export.py
        'mailchimp3~=3.0',
        'argon2-cffi~=21.3',
        'djangocms-aldryn-forms[captcha]',
        'djangocms-aldryn-search',
        'django-haystack~=3.2',
        'pymemcache~=4.0',
        'whoosh~=2.7',
    ],
    # Do not use test_require or build_require, because then it's not installed and is
    # able to be used only by setup.py util. We want to use it manually.
    # Actually it could be all in dev-requirements.txt but it's good to have it here
    # next to run dependencies and have it separated by purposes.
    extras_require={
        'dev': [
            'django-debug-toolbar~=4.1',
            'django-extensions~=3.2',
        ],
        'test': [
            'flake8',
            'isort',
            'mypy',
            'pylint',
            'pylint-django',
            'pytest~=6.2',
            'pytest-data==0.4',
            'pytest-django==3.9.0',
            'pytest-env==0.6.2',
            'pytest-pythonpath==0.7.3',
            'pytest-sugar==0.9.3',
            'pytest-watch==4.2.0',
            'PyVirtualDisplay==1.3.2',
            'webdriverwrapper==2.8.0',
            'django-simple-captcha==0.5.14',
            'testfixtures',
            'tzdata',
        ],
        'build': [
            'Jinja2<3.1.0',
            'Sphinx==1.8.5',
        ],
        'psql': [
            'psycopg2',
        ],
        'mysql': [
            'mysqlclient~=2.2',
        ],
        'newsblog': [
            'djangocms-aldryn-newsblog',
        ]
    },

    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Framework :: Django',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],
    keywords=['django', 'cms'],
)



import os
from pathlib import Path

site_resolver = Path(__file__).resolve()

PROJECT_DIR = site_resolver.parent.parent.parent.parent

HAYSTACK_ROUTERS = ['aldryn_search.router.LanguageRouter']
HAYSTACK_ENGINE = 'cms_qe.whoosh.backend.AnalyzerWhooshEngine'
_HAYSTACK_PATH = os.path.normpath(os.path.join(PROJECT_DIR, 'whoosh_index'))
HAYSTACK_CONNECTIONS = {
    'default': {'ENGINE': HAYSTACK_ENGINE, 'PATH': os.path.join(_HAYSTACK_PATH, 'default')},
    'en': {'ENGINE': HAYSTACK_ENGINE, 'PATH': os.path.join(_HAYSTACK_PATH, 'en')},
}
HAYSTACK_CUSTOM_HIGHLIGHTER = "cms_qe.haystack.highlighting.HaystackHighlighter"



import logging
import smtplib
import socket
from typing import Optional

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.utils.encoding import force_bytes, force_str
from django.utils.http import urlsafe_base64_decode, urlsafe_base64_encode

ACTION_OK = 250  # Requested mail action okay, completed.

logger = logging.getLogger(__name__)

# pylint:disable=invalid-name


def pk_to_uidb64(pk):
    return urlsafe_base64_encode(force_bytes(pk))


def uidb64_to_pk(uidb64):
    return force_str(urlsafe_base64_decode(uidb64))


def get_user_by_uidb64(uidb64):
    User = get_user_model()
    try:
        pk = uidb64_to_pk(uidb64)
        user = User.objects.get(pk=pk)
    except (TypeError, ValueError, OverflowError, User.DoesNotExist):
        user = None
    return user


class ActionIsNoOkResponse(Exception):
    """Action si not OK response."""


def check_smtp_response(response: tuple[int, bytes]) -> None:
    """Check if response is valid."""
    if response[0] != ACTION_OK:
        raise ActionIsNoOkResponse(force_str(response[1]))


class SMTPCheckRecipient:
    """SMTP server checks recipient email if the server accepts it."""

    def __init__(self, hostname: str):
        self.smtp: Optional[smtplib.SMTP] = None
        self.hostname = hostname

    def connect(self) -> None:
        if self.hostname and self.smtp is None:
            try:
                self.smtp = smtplib.SMTP(self.hostname, timeout=5)
                check_smtp_response(self.smtp.helo())
                check_smtp_response(self.smtp.docmd('MAIL FROM:""'))
            except (ActionIsNoOkResponse, OSError, socket.gaierror, smtplib.SMTPException) as error:
                logger.error(error)

    def close(self) -> None:
        if self.smtp is not None and self.smtp.sock is not None:
            self.smtp.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, exc_tb):
        self.close()

    def check(self, address: str) -> None:
        """Check if SMTP server accepts email address. Raise ValidationError if not."""
        self.connect()
        if self.smtp is not None and self.smtp.sock is not None:
            try:
                code, message = self.smtp.docmd(f'RCPT TO:{address}')
                if code != ACTION_OK:
                    raise ValidationError(force_str(message), code="invalid")
            except (socket.gaierror, smtplib.SMTPException) as error:
                logger.error(error)


def smtp_server_accepts_email_address(address: str) -> None:
    """SMTP server accepts email address. Raise ValidationError if not."""
    with SMTPCheckRecipient(settings.EMAIL_HOST) as checker:
        checker.check(address)



from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from .models import Iframe, Link, Script


@plugin_pool.register_plugin
class LinkTagPlugin(CMSPluginBase):

    model = Link
    name = 'LINK'
    module = 'HTML Elements'
    render_template = "cms_qe_plugins/link.html"


@plugin_pool.register_plugin
class ScriptTagPlugin(CMSPluginBase):

    model = Script
    name = 'SCRIPT'
    module = 'HTML Elements'
    render_template = "cms_qe_plugins/script.html"


@plugin_pool.register_plugin
class IframeTagPlugin(CMSPluginBase):

    model = Iframe
    name = 'IFRAME'
    module = 'HTML Elements'
    render_template = "cms_qe_plugins/iframe.html"



from django.contrib.auth import views as django_views
from django.urls import path

from cms_qe_auth.forms import PasswordResetFormWithEmailExistenceCheck

from . import views as cms_qe_auth_views

urlpatterns = [
    path('login/', django_views.LoginView.as_view(template_name='cms_qe/auth/login.html'), name='login'),
    path('logout/', django_views.LogoutView.as_view(template_name='cms_qe/auth/logged_out.html'), name='logout'),
    path('password_change/', django_views.PasswordChangeView.as_view(
        template_name='cms_qe/auth/password_change_form.html'), name='password_change'),
    path('password_change/done/', django_views.PasswordChangeDoneView.as_view(
        template_name='cms_qe/auth/password_change_done.html'), name='password_change_done'),
    path('password_reset/', django_views.PasswordResetView.as_view(
        template_name='cms_qe/auth/password_reset_form.html',
        email_template_name='cms_qe/auth/password_reset_email.txt',
        html_email_template_name='cms_qe/auth/password_reset_email.html',
        form_class=PasswordResetFormWithEmailExistenceCheck,
    ), name='password_reset'),
    path('password_reset/done/', django_views.PasswordResetDoneView.as_view(
        template_name='cms_qe/auth/password_reset_done.html'), name='password_reset_done'),
    path('reset/<slug:uidb64>/<slug:token>/', django_views.PasswordResetConfirmView.as_view(
        template_name='cms_qe/auth/password_reset_confirm.html'), name='password_reset_confirm'),
    path('reset/done/', django_views.PasswordResetCompleteView.as_view(
        template_name='cms_qe/auth/password_reset_complete.html'), name='password_reset_complete'),
    path('register/', cms_qe_auth_views.register, {'template_name': 'cms_qe/auth/register.html'}, name='register'),
    path('activate/<slug:uidb64>/<slug:token>/', cms_qe_auth_views.activate, {
        'template_name_complete': 'cms_qe/auth/email_confirmation_complete.html',
        'template_name_fail': 'cms_qe/auth/email_confirmation_fail.html',
    }, name='activate'),
]



from django.contrib.auth.tokens import PasswordResetTokenGenerator


class TokenGenerator(PasswordResetTokenGenerator):
    def _make_hash_value(self, user, timestamp):
        return f'{user.pk}{timestamp}{user.is_active}'



import pytest

from cms_qe_test import render_plugin

from .cms_plugins import BreadcrumbPlugin


@pytest.mark.skip(reason="Skip until django-cms >= 4.0.")
def test_render():
    """
    ValueError: invalid literal for int() with base 10: 'menu/breadcrumb.html'

    only_visible = 'menu/breadcrumb.html'

    menus/templatetags/menu_tags.py:
        try:
            only_visible = bool(int(only_visible))
        except TypeError:
            only_visible = bool(only_visible)
    """
    assert '<ol>' in render_plugin(BreadcrumbPlugin)
